/*
    SPDX-FileCopyrightText: 2024 Boris Pek <bpek@astralinux.ru>

    SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
*/

import QtQuick 2.6
import QtQuick.Layouts 1.3
import QtQuick.Controls 2.5 as QQC2
import QtGraphicalEffects 1.12
import org.kde.kirigami 2.12 as Kirigami

QQC2.Dialog {
    id: root

    property string text: ""

    function updateAndOpen() {
        inputField.text = root.text;
        root.open();
        inputField.forceActiveFocus();
    }

    modal: true
    standardButtons: QQC2.Dialog.Ok | QQC2.Dialog.Cancel
    height: mainColumn.implicitHeight +
            Kirigami.Units.largeSpacing * 2 +
            Kirigami.Units.smallSpacing * 2 +
            header.implicitHeight + footer.implicitHeight

    onOpened: inputField.forceActiveFocus()
    onRejected: {
        root.close();
        inputField.focus = false;
    }
    onAccepted: {
        if (inputField.text !== "") {
            root.text = inputField.text;
            root.close();
            inputField.focus = false;
        } else {
            root.open();
            inputField.forceActiveFocus();
        }
    }

    property int translateY: (1 - opacity) * Kirigami.Units.gridUnit * 2

    NumberAnimation on opacity {
        to: 1
        from: 0
        duration: Kirigami.Units.veryShortDuration
        easing.type: Easing.InOutQuad
        running: true
    }

    background: Item {
        transform: Translate { y: root.translateY }

        RectangularGlow {
            anchors.fill: rect
            anchors.topMargin: 1
            cornerRadius: rect.radius * 2
            glowRadius: 2
            spread: 0.2
            color: Qt.rgba(0, 0, 0, 0.3)
        }
        Rectangle {
            id: rect
            anchors.fill: parent
            Kirigami.Theme.inherit: false
            Kirigami.Theme.colorSet: Kirigami.Theme.Window
            color: Kirigami.Theme.backgroundColor
            radius: Kirigami.Units.smallSpacing

            Kirigami.Separator {
                id: topSeparator
                anchors.left: parent.left
                anchors.right: parent.right
                anchors.top: parent.top
                anchors.topMargin: root.header.implicitHeight
            }

            Kirigami.Separator {
                id: bottomSeparator
                anchors.left: parent.left
                anchors.right: parent.right
                anchors.bottom: parent.bottom
                anchors.bottomMargin: root.footer.implicitHeight
            }

            Rectangle {
                Kirigami.Theme.inherit: false
                Kirigami.Theme.colorSet: Kirigami.Theme.View
                color: Kirigami.Theme.backgroundColor
                anchors.left: parent.left
                anchors.right: parent.right
                anchors.top: topSeparator.bottom
                anchors.bottom: bottomSeparator.top
            }
        }
    }

    header: Item {
        transform: Translate { y: root.translateY }
        implicitHeight: heading.implicitHeight + Kirigami.Units.largeSpacing * 2

        Kirigami.Heading {
            id: heading
            level: 2
            text: i18n("Date format")
            wrapMode: Text.WordWrap
            anchors.left: parent.left
            anchors.right: parent.right
            anchors.leftMargin: Kirigami.Units.largeSpacing
            anchors.verticalCenter: parent.verticalCenter
        }
    }

    footer.transform: Translate { y: root.translateY }

    ColumnLayout {
        id: mainColumn
        spacing: Kirigami.Units.smallSpacing
        implicitHeight: Layout.minimumHeight

        ColumnLayout {
            Layout.preferredWidth: Kirigami.Units.gridUnit * 20
            Layout.alignment: Qt.AlignHCenter

            Kirigami.InlineMessage {
                id: infoMessage
                Layout.fillWidth: true
                type: Kirigami.MessageType.Information
                text: i18n("<a href=\"https://doc.qt.io/qt-5/qml-qtqml-qt.html#formatDateTime-method\">Time Format Documentation</a>")
                visible: true

                onLinkActivated: Qt.openUrlExternally(link)
                MouseArea {
                    anchors.fill: parent
                    acceptedButtons: Qt.NoButton // We don't want to eat clicks on the Label
                    cursorShape: parent.hoveredLink ? Qt.PointingHandCursor : Qt.ArrowCursor
                }
            }
            Kirigami.ActionTextField {
                id: inputField
                Layout.fillWidth: true
            }
        }
    }
}
