/*
    SPDX-FileCopyrightText: 2013 Bhushan Shah <bhush94@gmail.com>
    SPDX-FileCopyrightText: 2013 Sebastian Kügler <sebas@kde.org>
    SPDX-FileCopyrightText: 2015 Kai Uwe Broulik <kde@privat.broulik.de>
    SPDX-FileCopyrightText: 2023 Boris Pek <bpek@astralinux.ru>

    SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
*/

import QtQuick 2.6
import QtQuick.Layouts 1.15
import QtQuick.Controls 2.15 as QtControls
import org.kde.kquickcontrolsaddons 2.0 // For KCMShell
import org.kde.kcm 1.2 as KCM
import org.kde.kirigamiaddons.labs.mobileform 0.1 as MobileForm
import org.kde.plasma.plasmoid 2.0
import org.kde.plasma.core 2.0 as PlasmaCore

KCM.SimpleKCM {
    id: appearancePage

    property bool cfg_autoFontAndSize: true
    property int cfg_dateDisplayFormat: 2

    // boldText and fontStyleName are not used in DigitalClock.qml
    // However, they are necessary to remember the exact font style chosen.
    // Otherwise, when the user open the font dialog again, the style will be lost.
    property string cfg_fontFamily: ""
    property string cfg_boldText: ""
    property string cfg_italicText: ""
    property string cfg_fontWeight: ""
    property string cfg_fontStyleName: ""
    property string cfg_fontSize: ""

    property string cfg_timeFormat: ""
    property bool cfg_showLocalTimezone: false
    property alias cfg_displayTimezoneFormat: displayTimezoneFormat.currentIndex
    property alias cfg_showSeconds: showSeconds.checked

    property alias cfg_showDate: showDate.checked
    property string cfg_dateFormat: "shortDate"
    property alias cfg_customDateFormat: customDateFormatDialog.text
    property alias cfg_use24hFormat: use24hFormat.currentIndex

    ColumnLayout {
        Layout.fillWidth: true

        MobileForm.FormCard {
            Layout.fillWidth: true
            contentItem: ColumnLayout {
                // Kirigami.FormData.label: i18n("Information:")
                MobileForm.FormSwitchDelegate {
                    id: showDate
                    text: i18n("Show date")
                }
                MobileForm.FormSwitchDelegate {
                    id: showSeconds
                    text: i18n("Show seconds")
                }

                MobileForm.FormComboBoxDelegate {
                    id: showTimezone
                    text: i18n("Show time zone")
                    model: [
                        i18n("Only when different from local time zone"),
                        i18n("Always")
                    ]
                    onCurrentIndexChanged: cfg_showLocalTimezone = (currentIndex === 1)
                }

                MobileForm.FormComboBoxDelegate {
                    id: displayTimezoneFormat
                    text: i18n("Display time zone as")
                    model: [
                        i18n("Code"),
                        i18n("City"),
                        i18n("Offset from UTC time"),
                    ]
                    onActivated: cfg_displayTimezoneFormat = currentIndex
                }

                MobileForm.FormComboBoxDelegate {
                    id: use24hFormat
                    text: i18n("Time display")
                    model: [
                        i18n("12-Hour"),
                        i18n("Use Region Defaults"),
                        i18n("24-Hour")
                    ]
                    onCurrentIndexChanged: cfg_use24hFormat = currentIndex
                }
                MobileForm.FormComboBoxDelegate {
                    id: dateFormat
                    enabled: showDate.checked
                    description: Qt.formatDate(new Date(), cfg_dateFormat === "custom" ? cfg_customDateFormat
                                                                                       : dateFormat.model[dateFormat.currentIndex].format)
                    text: i18n("Date format")
                    textRole: "label"
                    model: [
                        {
                            'label': i18n("Long Date"),
                            'name': "longDate",
                            format: Qt.SystemLocaleLongDate
                        },
                        {
                            'label': i18n("Short Date"),
                            'name': "shortDate",
                            format: Qt.SystemLocaleShortDate
                        },
                        {
                            'label': i18n("ISO Date"),
                            'name': "isoDate",
                            format: Qt.ISODate
                        },
                        {
                            'label': i18nc("custom date format", "Custom"),
                            'name': "custom"
                        }
                    ]
                    onActivated: {
                        cfg_dateFormat = model[currentIndex]["name"]
                        if (cfg_dateFormat === "custom") {
                            customDateFormatDialog.text = cfg_customDateFormat;
                            customDateFormatDialog.updateAndOpen();
                        }
                    }

                    Component.onCompleted: {
                        for (var i = 0; i < model.length; i++) {
                            if (model[i]["name"] === Plasmoid.configuration.dateFormat) {
                                dateFormat.currentIndex = i;
                                break;
                            }
                        }
                    }
                }

                MobileForm.FormButtonDelegate {
                    visible: KCMShell.authorize("kcm_regionandlang.desktop").length > 0
                    text: i18n("Change Regional Settings…")
                    icon.name: "preferences-desktop-locale"
                    onClicked: KCMShell.openSystemSettings("kcm_regionandlang")
                }
            }
        }
    }

    DateFormatDialog {
        id: customDateFormatDialog;
        x: (parent.width - width) / 2
    }

    Component.onCompleted: {
        if (Plasmoid.configuration.showLocalTimezone) {
            showTimezone.currentIndex = 1;
        } else {
            showTimezone.currentIndex = 0;
        }
    }
}
